@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006
+ * Copyright (c) 2006, 2007
  * Nintendo Co., Ltd.
  *
  * Permission to use, copy, modify, distribute and sell this software
@@ -25,6 +25,12 @@
 using namespace ATAttachment;
 using namespace Register;
 
+bool AtaController::
+isAtaDevice(const u8* signature)
+{
+    return (signature[2] == 0x00 && signature[3] == 0x00) ? true : false;
+}
+
 bool AtaController::
 isAtapiDevice(const u8* signature)
 {
@@ -97,13 +103,23 @@ sync(u8 status)
 {
     using namespace Status;
 
+    int timeout = 0;
     while (inpb(ctlPort + ALTERNATE_STATUS) & BSY)
     {
 #if defined(__i386__) || defined(__x86_64__)
         __asm__ __volatile__ ("pause\n");
+        // Reading or writing a byte from/to port 0x80 take almost
+        // exactly 1 microsecond independent of the processor type
+        // and speed.
+        __asm__ __volatile__ ("outb %%al, $0x80" ::: "%eax");
 #endif
+        if (1000000L < ++timeout)
+        {
+            esReport("AtaController::sync() -- timeout\n");
+            break;
+        }
     }
-    // It seems ALTERNATE_STATUS is not properly emulated under QEMU except the BSY bit.
+    inpb(ctlPort + ALTERNATE_STATUS);
     return inpb(cmdPort + STATUS);
 }
 
@@ -118,6 +134,10 @@ issue(AtaDevice* device, u8 cmd, void* buffer, int count, long long lba)
     using namespace Command;
     using namespace Status;
 
+#ifdef VERBOSE
+    esReport("AtaController::%s(%02x, %p, %d, %lld)\n", __func__, cmd, buffer, count, lba);
+#endif
+
     Lock::Synchronized io(lock);
 
     if (device->id[FEATURES_COMMAND_SETS_SUPPORTED + 1] & 0x0400)
@@ -315,7 +335,7 @@ invoke(int param)
 #ifdef VERBOSE
     if (param)
     {
-        esReport("AtaController::%s(%d) : %02x : %02x\n", __func__, param, this->cmd, status);
+        esReport("AtaController::%s(%d) : %02xh : %02xh\n", __func__, param, cmd, status);
     }
 #endif
 
@@ -338,6 +358,9 @@ invoke(int param)
                 {
                     done = true;
                 }
+#ifdef VERBOSE
+                esReport("AtaController::%s : %p, %p, %d\n", __func__, data, limit, done);
+#endif
             }
             else
             {
@@ -359,6 +382,9 @@ invoke(int param)
                 {
                     done = true;
                 }
+#ifdef VERBOSE
+                esReport("AtaController::%s : %d, %p, %p, %d\n", __func__, len, data, limit, done);
+#endif
             }
             else
             {
@@ -479,6 +505,8 @@ AtaController(int cmdPort, int ctlPort, int irq, AtaDma* dma, IContext* ata) :
     dma(dma),
     thread(run, this, IThread::Highest - (irq - 14))
 {
+    device[0] = device[1] = 0;
+
     esCreateInstance(CLSID_Monitor,
                      IID_IMonitor,
                      reinterpret_cast<void**>(&monitor));
@@ -495,11 +523,11 @@ AtaController(int cmdPort, int ctlPort, int irq, AtaDma* dma, IContext* ata) :
         esReport("device0: signature %02x %02x %02x %02x %02x\n",
                  signature[0], signature[1], signature[2], signature[3], signature[4]);
 #endif
-        if (!isAtapiDevice(signature))
+        if (isAtaDevice(signature))
         {
             device[0] = new AtaDevice(this, Device::MASTER, signature);
         }
-        else
+        else if (isAtapiDevice(signature))
         {
             device[0] = new AtaPacketDevice(this, Device::MASTER, signature);
         }
@@ -511,17 +539,22 @@ AtaController(int cmdPort, int ctlPort, int irq, AtaDma* dma, IContext* ata) :
         esReport("device1: signature %02x %02x %02x %02x %02x\n",
                  signature[0], signature[1], signature[2], signature[3], signature[4]);
 #endif
-        if (!isAtapiDevice(signature))
+        if (isAtaDevice(signature))
         {
             device[1] = new AtaDevice(this, Device::SLAVE, signature);
         }
-        else
+        else if (isAtapiDevice(signature))
         {
             device[1] = new AtaPacketDevice(this, Device::SLAVE, signature);
         }
     }
 
-    Core::registerExceptionHandler(32 + irq, this);
+    if (device[0] == 0 && device[1] == 0)
+    {
+        return;
+    }
+
+    Core::registerInterruptHandler(irq, this);
 
     for (int i = 0; i < 10; ++i)
     {
