@@ -12,9 +12,13 @@
  */
 
 #include <string.h> // ffs()
+#include "apic.h"
+#include "core.h"
 #include "thread.h"
 #include "process.h"
 
+Ref Sched::numCores(0);
+
 Sched::
 Sched() :
     runQueueBits(0),
@@ -26,14 +30,12 @@ void Sched::
 setRun(Thread* thread)
 {
     lock();
-    ASSERT(thread->state == IThread::RUNNABLE);
+    thread->state = IThread::RUNNABLE;
     Thread::Queue* queue = &runQueue[thread->priority];
     queue->addLast(thread);
     runQueueBits |= 0x80000000u >> thread->priority;
     runQueueHint = true;   // Hint to scheduler to check run queue
     unlock();
-
-    // XXX IPI to notify the other processors that another thread becomes ready to run.
 }
 
 void Sched::
@@ -61,14 +63,14 @@ selectThread()
     {
         do
         {
-            unsigned x = Thread::splIdle();
+            unsigned x = Core::splIdle();
             while (runQueueBits == 0)
             {
 #ifdef __i386__
                 __asm__ __volatile__ ("hlt\n");
 #endif
             }
-            Thread::splX(x);
+            Core::splX(x);
         } while (runQueueBits == 0);
 
         lock();
@@ -85,16 +87,21 @@ selectThread()
         unlock();
     }
 
+    ASSERT(next->state == IThread::RUNNABLE);
     ASSERT(next->priority == priority);
     queue->remove(next);
+    ASSERT(!queue->contains(next));
     if (queue->isEmpty())
     {
         runQueueBits &= ~(0x80000000u >> priority);
     }
     next->state = IThread::RUNNING;
+    next->core = Core::getCurrentCore();
 
     unlock();
+    ASSERT(next->checkStack());
     next->unlock();     // XXX check if we can unlock next now
+
     return next;
 }
 
@@ -258,6 +265,17 @@ setFocus(void* (*focus)(void* param))
     return current->setFocus(focus);
 }
 
+//
+// ICallback
+//
+
+int Sched::
+invoke(int result)
+{
+    // Process IPIs
+    int vec = 32 + result;
+}
+
 //
 // IInterface
 //
@@ -277,6 +295,10 @@ queryInterface(const Guid& riid, void** objectPtr)
     {
         *objectPtr = static_cast<IRuntime*>(this);
     }
+    else if (riid == IID_ICallback)
+    {
+        *objectPtr = static_cast<ICallback*>(this);
+    }
     else if (riid == IID_IInterface)
     {
         *objectPtr = static_cast<ICurrentThread*>(this);
