@@ -24,7 +24,8 @@
 
 class InternetConfig : public IInternetConfig
 {
-    Ref     ref;
+    Ref                             ref;
+    AddressSet<IInternetAddress>    nameServers;
 
 public:
     //
@@ -38,9 +39,14 @@ public:
     IInternetAddress* getRouter();
     void removeRouter(IInternetAddress* router);
 
-    int addInterface(IStream* stream, int hrd);
+    int addInterface(INetworkInterface* networkInterface);
     IInterface* getInterface(int scopeID);
-    void removeInterface(IStream* stream);
+    int getScopeID(INetworkInterface* networkInterface);
+    void removeInterface(INetworkInterface* networkInterface);
+
+    void addNameServer(IInternetAddress* address);
+    IInternetAddress* getNameServer();
+    void removeNameServer(IInternetAddress* address);
 
     //
     // IInterface
